﻿Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class MFPlayer
    Implements IDisposable

#Region "Variablen"
    Private m_MFInit As Boolean = False
    Private m_RenderHwnd As IntPtr = IntPtr.Zero
    Private m_DisposedValue As Boolean

    Private m_IMFClock As IMFClock
    Private m_IMFMediaSource As IMFMediaSource
    Private m_IMFMediaSession As IMFMediaSession
    Private m_IMFAudioStreamVolume As IMFAudioStreamVolume
    Private m_IMFPresentationDescriptor As IMFPresentationDescriptor
#End Region

#Region "Class"
    Public Sub New()

        If m_MFInit = False Then

            If MFStartup(MF_VERSION_2, MFSTARTUP_FULL) = S_OK Then

                m_MFInit = True

            End If

        End If

    End Sub
#End Region

#Region "Properties"
    Public ReadOnly Property IsMFInit As Boolean
        Get
            Return m_MFInit
        End Get
    End Property
#End Region

#Region "Function"
    Public Function GetStreamVolumeChannelCount() As Integer

        Dim intRet As Integer
        Dim intChannelCount As Integer

        If m_IMFAudioStreamVolume IsNot Nothing Then

            If m_IMFAudioStreamVolume.GetChannelCount(intChannelCount) = S_OK Then

                intRet = intChannelCount

            End If

        End If

        GetStreamVolumeChannelCount = intRet

    End Function

    Public Function GetStreamVolume(ByVal intChannelIndex As Integer) As Single

        Dim sngRet As Single
        Dim sngChannelVolume As Single
        Dim intChannelCount As Integer

        If m_IMFAudioStreamVolume IsNot Nothing Then

            intChannelCount = GetStreamVolumeChannelCount()

            If intChannelIndex < 0 Then intChannelIndex = 0
            If intChannelIndex > intChannelCount - 1 Then intChannelIndex = intChannelCount - 1

            If m_IMFAudioStreamVolume.GetChannelVolume(intChannelIndex, sngChannelVolume) = S_OK Then

                sngRet = sngChannelVolume

            End If

        End If

        GetStreamVolume = sngRet

    End Function

    Public Function SetStreamVolume(ByVal intChannelIndex As Integer, ByVal sngChannelVolume As Single) As Boolean

        Dim bolRet As Boolean = False
        Dim intChannelCount As Integer

        If m_IMFAudioStreamVolume IsNot Nothing Then

            intChannelCount = GetStreamVolumeChannelCount()

            If intChannelIndex < 0 Then intChannelIndex = 0
            If intChannelIndex > intChannelCount - 1 Then intChannelIndex = intChannelCount - 1

            If sngChannelVolume < 0 Then sngChannelVolume = 0
            If sngChannelVolume > 1 Then sngChannelVolume = 1

            If m_IMFAudioStreamVolume.SetChannelVolume(intChannelIndex, sngChannelVolume) = S_OK Then

                bolRet = True

            End If

        End If

        SetStreamVolume = bolRet

    End Function

    Public Sub SetRenderHwnd(ByVal hWnd As IntPtr)

        m_RenderHwnd = hWnd

    End Sub

    Public Function PlayMedia() As Boolean

        Dim bolRet As Boolean = False

        If m_IMFMediaSession IsNot Nothing Then

            If m_IMFMediaSession.Start(New Guid(GUID_NULL), Nothing) = S_OK Then

                bolRet = True

            End If

        End If

        PlayMedia = bolRet

    End Function

    Public Function PauseMedia() As Boolean

        Dim bolRet As Boolean = False

        If m_IMFMediaSession IsNot Nothing Then

            If m_IMFMediaSession.Pause = S_OK Then

                bolRet = True

            End If

        End If

        PauseMedia = bolRet

    End Function

    Public Function StopMedia() As Boolean

        Dim bolRet As Boolean = False

        If m_IMFMediaSession IsNot Nothing Then

            If m_IMFMediaSession.Stop = S_OK Then

                bolRet = True

            End If

        End If

        StopMedia = bolRet

    End Function

    Public Function GetMediaPlayPosition() As ULong

        Dim ulngRet As ULong
        Dim ulngClockTime As ULong
        Dim ulngSystemTime As ULong

        If m_IMFClock IsNot Nothing Then

            If m_IMFClock.GetCorrelatedTime(0, ulngClockTime, ulngSystemTime) = S_OK Then

                ulngRet = ulngClockTime

            End If

        End If

        GetMediaPlayPosition = ulngRet

    End Function

    Public Function GetMediaDuration() As ULong

        Dim ulngRet As ULong
        Dim ulngDuaration As ULong

        If m_IMFPresentationDescriptor IsNot Nothing Then

            If m_IMFPresentationDescriptor.GetUINT64(New Guid(MF_PD_DURATION), ulngDuaration) = S_OK Then

                ulngRet = ulngDuaration

            End If

        End If

        GetMediaDuration = ulngRet

    End Function

    Public Function SetMediaUrl(ByVal strURL As String) As Boolean

        Dim bolRet As Boolean = False

        If m_MFInit Then

            CloseMediaSession()

            If CreateMediaSourceFromUrl(strURL, m_IMFMediaSource) Then

                If CreatePlaybackTopology(m_IMFMediaSource,
                                          m_IMFMediaSession,
                                          m_IMFPresentationDescriptor,
                                          m_IMFClock) Then

                    CreateServices(m_IMFMediaSource, m_IMFMediaSession, m_IMFPresentationDescriptor)

                    bolRet = True


                Else

                    CloseMediaSession()

                End If

            Else

                CloseMediaSession()

            End If

        End If

        SetMediaUrl = bolRet

    End Function

    Private Sub CreateServices(ByVal oIMFMediaSource As IMFMediaSource,
                               ByVal oIMFMediaSession As IMFMediaSession,
                               ByVal oIMFPresentationDescriptor As IMFPresentationDescriptor)

        ' Normalerweise würde man das Media Foundation Event-Interface
        ' IMFMediaEventGenerator verwenden um ein entsprechendes Event zu
        ' erhalten wann die MediaSession fertig erstellt wurde.
        ' Ich warte hier halt 100 Millisekunden da ich für dieses Beispiel ohne Events arbeite.
        Threading.Thread.Sleep(100)

        ' weitere Services die auf die Interfaces IMFMediaSource, IMFMediaSession
        ' und IMFPresentationDescriptor angewendet werden können.
        ' https://docs.microsoft.com/en-us/windows/win32/medfound/service-interfaces

        Dim MFAudioStreamVolume As Object = Nothing

        If MFGetService(oIMFMediaSession, New Guid(MR_STREAM_VOLUME_SERVICE),
                        New Guid(IID_IMFAudioStreamVolume), MFAudioStreamVolume) = S_OK Then

            m_IMFAudioStreamVolume = DirectCast(MFAudioStreamVolume, IMFAudioStreamVolume)

        End If

    End Sub

    Private Sub CloseMediaSession()

        If m_IMFMediaSession IsNot Nothing Then

            m_IMFMediaSession.Stop()
            m_IMFMediaSession.Close()

        End If

        If m_IMFMediaSource IsNot Nothing Then

            If m_IMFMediaSource.Shutdown = S_OK Then

                Marshal.ReleaseComObject(m_IMFMediaSource)

            End If

        End If

        If m_IMFClock IsNot Nothing Then

            Marshal.ReleaseComObject(m_IMFClock)

        End If

        If m_IMFMediaSession IsNot Nothing Then

            If m_IMFMediaSession.Shutdown = S_OK Then

                Marshal.ReleaseComObject(m_IMFMediaSession)

            End If

        End If

        If m_IMFPresentationDescriptor IsNot Nothing Then

            Marshal.ReleaseComObject(m_IMFPresentationDescriptor)

        End If

        If m_IMFAudioStreamVolume IsNot Nothing Then

            Marshal.ReleaseComObject(m_IMFAudioStreamVolume)

        End If

    End Sub

    Private Function CreateMediaSourceFromUrl(ByVal strURL As String,
                                              ByRef oIMFMediaSource As IMFMediaSource) As Boolean

        Dim bolRet As Boolean = False
        Dim eMFOT As MF_OBJECT_TYPE
        Dim pIUnknown As IntPtr = IntPtr.Zero
        Dim pIMFMediaSource As IntPtr = IntPtr.Zero
        Dim MFMediaSource As IMFMediaSource
        Dim MFSourceResolver As IMFSourceResolver = Nothing

        If MFCreateSourceResolver(MFSourceResolver) = S_OK Then

            eMFOT = MF_OBJECT_TYPE.MF_OBJECT_INVALID

            If MFSourceResolver.CreateObjectFromURL(strURL, MF_RESOLUTION_TYPE.MF_RESOLUTION_MEDIASOURCE Or
                                                    MF_RESOLUTION_TYPE.MF_RESOLUTION_CONTENT_DOES_NOT_HAVE_TO_MATCH_EXTENSION_OR_MIME_TYPE,
                                                    IntPtr.Zero, eMFOT, pIUnknown) = S_OK Then

                If eMFOT = MF_OBJECT_TYPE.MF_OBJECT_MEDIASOURCE Then

                    If Marshal.QueryInterface(pIUnknown, New Guid(IID_IMFMediaSource), pIMFMediaSource) = S_OK Then

                        MFMediaSource = DirectCast(Marshal.GetObjectForIUnknown(pIMFMediaSource), IMFMediaSource)

                        If MFMediaSource IsNot Nothing Then

                            oIMFMediaSource = MFMediaSource

                            bolRet = True

                        End If

                    End If

                End If

                Marshal.Release(pIUnknown)

            End If

            Marshal.ReleaseComObject(MFSourceResolver)

        End If

        CreateMediaSourceFromUrl = bolRet

    End Function

    Private Function CreatePlaybackTopology(ByVal oIMFMediaSource As IMFMediaSource,
                                            ByRef oIMFMediaSession As IMFMediaSession,
                                            ByRef oIMFPresentationDescriptor As IMFPresentationDescriptor,
                                            ByRef oIMFClock As IMFClock) As Boolean

        Dim bolRet As Boolean = False
        Dim intDescriptorCount As Integer
        Dim MFClock As IMFClock = Nothing
        Dim MFTopology As IMFTopology = Nothing
        Dim MFMediaSession As IMFMediaSession = Nothing
        Dim MFPresentationDescriptor As IMFPresentationDescriptor = Nothing

        If oIMFMediaSource.CreatePresentationDescriptor(MFPresentationDescriptor) = S_OK Then

            If MFCreateTopology(MFTopology) = S_OK Then

                If MFPresentationDescriptor.GetStreamDescriptorCount(intDescriptorCount) = S_OK Then

                    If intDescriptorCount > 0 Then

                        For intDescriptorItem As Integer = 0 To intDescriptorCount - 1

                            bolRet = AddBranchToPartialTopology(oIMFMediaSource,
                                                                intDescriptorItem,
                                                                MFPresentationDescriptor,
                                                                MFTopology)

                        Next

                        If bolRet Then

                            bolRet = False

                            If MFCreateMediaSession(IntPtr.Zero, MFMediaSession) = S_OK Then

                                If MFMediaSession.SetTopology(0, MFTopology) = S_OK Then

                                    If MFMediaSession.GetClock(MFClock) = S_OK Then

                                        bolRet = True

                                    End If

                                End If

                                If bolRet Then

                                    oIMFClock = MFClock

                                    oIMFMediaSession = MFMediaSession

                                    oIMFPresentationDescriptor = MFPresentationDescriptor

                                Else

                                    Marshal.ReleaseComObject(MFClock)

                                    Marshal.ReleaseComObject(MFMediaSession)

                                    Marshal.ReleaseComObject(MFPresentationDescriptor)

                                End If

                            End If

                        End If

                    End If

                End If

                Marshal.ReleaseComObject(MFTopology)

            End If

        End If

        CreatePlaybackTopology = bolRet

    End Function

    Private Function AddBranchToPartialTopology(ByVal oIMFMediaSource As IMFMediaSource,
                                                ByVal intIndex As Integer,
                                                ByVal oIMFPresentationDescriptor As IMFPresentationDescriptor,
                                                ByVal oIMFTopology As IMFTopology) As Boolean

        Dim bolRet As Boolean = False
        Dim bolSelected As Boolean = False
        Dim MFTopologyNode_Source As IMFTopologyNode = Nothing
        Dim MFTopologyNode_Output As IMFTopologyNode = Nothing
        Dim MFStreamDescriptor As IMFStreamDescriptor = Nothing

        If oIMFPresentationDescriptor.GetStreamDescriptorByIndex(intIndex,
                                                                 bolSelected,
                                                                 MFStreamDescriptor) = S_OK Then

            If bolSelected Then

                If CreateSourceNode(oIMFMediaSource, oIMFPresentationDescriptor,
                                    MFStreamDescriptor, MFTopologyNode_Source) Then

                    If CreateOutputNode(MFStreamDescriptor, MFTopologyNode_Output) Then

                        If oIMFTopology.AddNode(MFTopologyNode_Source) = S_OK Then

                            If oIMFTopology.AddNode(MFTopologyNode_Output) = S_OK Then

                                If MFTopologyNode_Source.ConnectOutput(0, MFTopologyNode_Output, 0) = S_OK Then

                                    bolRet = True

                                End If

                            End If

                        End If

                        Marshal.ReleaseComObject(MFTopologyNode_Output)

                    End If

                    Marshal.ReleaseComObject(MFTopologyNode_Source)

                End If

            End If

            Marshal.ReleaseComObject(MFStreamDescriptor)

        End If

        AddBranchToPartialTopology = bolRet

    End Function

    Private Function CreateSourceNode(ByVal oIMFMediaSource As IMFMediaSource,
                                      ByVal oIMFPresentationDescriptor As IMFPresentationDescriptor,
                                      ByVal oIMFStreamDescriptor As IMFStreamDescriptor,
                                      ByRef oMFTopologyNode As IMFTopologyNode) As Boolean

        Dim bolRet As Boolean = False
        Dim MFTopologyNode As IMFTopologyNode = Nothing

        If MFCreateTopologyNode(MF_TOPOLOGY_TYPE.MF_TOPOLOGY_SOURCESTREAM_NODE,
                                MFTopologyNode) = S_OK Then

            If MFTopologyNode.SetUnknown(New Guid(MF_TOPONODE_SOURCE),
                                         oIMFMediaSource) = S_OK Then

                If MFTopologyNode.SetUnknown(New Guid(MF_TOPONODE_PRESENTATION_DESCRIPTOR),
                                             oIMFPresentationDescriptor) = S_OK Then

                    If MFTopologyNode.SetUnknown(New Guid(MF_TOPONODE_STREAM_DESCRIPTOR),
                                                 oIMFStreamDescriptor) = S_OK Then

                        bolRet = True

                    End If

                End If

            End If

            If bolRet Then

                oMFTopologyNode = MFTopologyNode

            Else

                Marshal.ReleaseComObject(MFTopologyNode)

            End If

        End If

        CreateSourceNode = bolRet

    End Function

    Private Function CreateOutputNode(ByVal oIMFStreamDescriptor As IMFStreamDescriptor,
                                      ByRef oMFTopologyNode As IMFTopologyNode) As Boolean

        Dim bolRet As Boolean = False
        Dim tGUID As Guid
        Dim MFActivate As IMFActivate = Nothing
        Dim MFTopologyNode As IMFTopologyNode = Nothing
        Dim MFMediaTypeHandler As IMFMediaTypeHandler = Nothing

        If oIMFStreamDescriptor.GetMediaTypeHandler(MFMediaTypeHandler) = S_OK Then

            If MFMediaTypeHandler.GetMajorType(tGUID) = S_OK Then

                If tGUID.ToString = MFMediaType_Audio Then

                    If MFCreateAudioRendererActivate(MFActivate) <> S_OK Then

                        Marshal.ReleaseComObject(MFMediaTypeHandler)

                    Else

                        bolRet = True

                    End If

                ElseIf tGUID.ToString = MFMediaType_Video Then

                    If m_RenderHwnd <> IntPtr.Zero Then

                        If MFCreateVideoRendererActivate(m_RenderHwnd,
                                                         MFActivate) <> S_OK Then

                            Marshal.ReleaseComObject(MFMediaTypeHandler)

                        Else

                            bolRet = True

                        End If

                    End If

                End If

                If bolRet Then

                    bolRet = False

                    If MFCreateTopologyNode(MF_TOPOLOGY_TYPE.MF_TOPOLOGY_OUTPUT_NODE,
                                            MFTopologyNode) = S_OK Then

                        If MFTopologyNode.SetObject(MFActivate) = S_OK Then

                            bolRet = True

                        End If

                        If bolRet Then

                            oMFTopologyNode = MFTopologyNode

                        Else

                            Marshal.ReleaseComObject(MFTopologyNode)

                        End If

                    End If

                End If

                If MFActivate.ShutdownObject = S_OK Then

                    Marshal.ReleaseComObject(MFActivate)

                End If

            End If

            Marshal.ReleaseComObject(MFMediaTypeHandler)

        End If

        CreateOutputNode = bolRet

    End Function

    Public Function Time2String(ByVal ulngTime As ULong) As String

        Dim t As TimeSpan = TimeSpan.FromMilliseconds(ulngTime / 10000)

        Time2String = String.Format("{0:D2}h:{1:D2}m:{2:D2}s:{3:D3}ms",
                                    t.Hours, t.Minutes, t.Seconds, t.Milliseconds)

    End Function

#End Region

#Region "IDisposable"
    Protected Overridable Sub Dispose(disposing As Boolean)

        If Not m_DisposedValue Then

            If disposing Then

                If m_MFInit Then

                    CloseMediaSession()

                    If MFShutdown() = S_OK Then

                        m_MFInit = False

                    End If

                End If

            End If

            m_DisposedValue = True

        End If

    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose

        Dispose(disposing:=True)

        GC.SuppressFinalize(Me)

    End Sub

    Protected Overrides Sub Finalize()

        MyBase.Finalize()

    End Sub
#End Region

#Region "API"
    <DllImport("Mfplat.dll", EntryPoint:="MFStartup")>
    <PreserveSig> Private Shared Function MFStartup(<[In]> ByVal Version As Integer,
                                                    <[In]> ByVal dwFlags As Integer) As Integer
    End Function

    <DllImport("Mfplat.dll", EntryPoint:="MFShutdown")>
    <PreserveSig> Private Shared Function MFShutdown() As Integer
    End Function

    <DllImport("Mfplat.dll", EntryPoint:="MFCreateSourceResolver")>
    <PreserveSig> Private Shared Function MFCreateSourceResolver(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppISourceResolver As IMFSourceResolver) As Integer
    End Function

    <DllImport("Mf.dll", EntryPoint:="MFCreateTopology")>
    <PreserveSig> Private Shared Function MFCreateTopology(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppTopo As IMFTopology) As Integer
    End Function

    <DllImport("Mf.dll", EntryPoint:="MFCreateMediaSession")>
    <PreserveSig> Private Shared Function MFCreateMediaSession(<[In]> ByVal pConfiguration As IntPtr,
                                                               <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppMediaSession As IMFMediaSession) As Integer
    End Function

    <DllImport("Mf.dll", EntryPoint:="MFCreateAudioRendererActivate")>
    <PreserveSig> Private Shared Function MFCreateAudioRendererActivate(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppActivate As IMFActivate) As Integer
    End Function

    <DllImport("Mf.dll", EntryPoint:="MFCreateVideoRendererActivate")>
    <PreserveSig> Private Shared Function MFCreateVideoRendererActivate(<[In]> ByVal hwndVideo As IntPtr,
                                                                        <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppActivate As IMFActivate) As Integer
    End Function

    <DllImport("Mf.dll", EntryPoint:="MFCreateTopologyNode")>
    <PreserveSig> Private Shared Function MFCreateTopologyNode(<[In]> ByVal NodeType As MF_TOPOLOGY_TYPE,
                                                               <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppNode As IMFTopologyNode) As Integer
    End Function

    <DllImport("Mf.dll", EntryPoint:="MFGetService")>
    <PreserveSig> Private Shared Function MFGetService(<[In], MarshalAs(UnmanagedType.Interface)> ByVal punkObject As Object,
                                                       <[In]> ByRef guidService As Guid,
                                                       <[In]> ByRef riid As Guid,
                                                       <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppvObject As Object) As Integer
    End Function
#End Region

#Region "Const"
    Private Const S_OK As Integer = 0
    Private Const MF_VERSION_2 As Integer = &H20070
    Private Const MFSTARTUP_FULL As Integer = 0

    Private Const GUID_NULL As String = "00000000-0000-0000-0000-000000000000"

    Private Const MF_PD_DURATION As String = "6c990d33-bb8e-477a-8598-0d5d96fcd88a"

    Private Const MR_STREAM_VOLUME_SERVICE As String = "f8b5fa2f-32ef-46f5-b172-1321212fb2c4"

    Private Const MFMediaType_Audio As String = "73647561-0000-0010-8000-00aa00389b71"
    Private Const MFMediaType_Video As String = "73646976-0000-0010-8000-00aa00389b71"

    Private Const MF_TOPONODE_SOURCE As String = "835c58ec-e075-4bc7-bcba-4de000df9ae6"
    Private Const MF_TOPONODE_STREAM_DESCRIPTOR As String = "835c58ee-e075-4bc7-bcba-4de000df9ae6"
    Private Const MF_TOPONODE_PRESENTATION_DESCRIPTOR As String = "835c58ed-e075-4bc7-bcba-4de000df9ae6"

    Private Const IID_IMFClock As String = "2eb1e945-18b8-4139-9b1a-d5d584818530"
    Private Const IID_IMFActivate As String = "7fee9e9a-4a89-47a6-899c-b6a53a70fb67"
    Private Const IID_IMFTopology As String = "83CF873A-F6DA-4bc8-823F-BACFD55DC433"
    Private Const IID_IMFAttributes As String = "2cd2d921-c447-44a7-a13c-4adabfc247e3"
    Private Const IID_IMFMediaSource As String = "279a808d-aec7-40c8-9c6b-a6b492c78a66"
    Private Const IID_IMFTopologyNode As String = "83CF873A-F6DA-4bc8-823F-BACFD55DC430"
    Private Const IID_IMFMediaSession As String = "90377834-21D0-4dee-8214-BA2E3E6C1127"
    Private Const IID_IMFSourceResolver As String = "FBE5A32D-A497-4b61-BB85-97B1A848A6E3"
    Private Const IID_IMFMediaTypeHandler As String = "e93dcf6c-4b07-4e1e-8123-aa16ed6eadf5"
    Private Const IID_IMFStreamDescriptor As String = "56c03d9c-9dbb-45f5-ab4b-d80f47c05938"
    Private Const IID_IMFAudioStreamVolume As String = "76b1bbdb-4ec8-4f36-b106-70a9316df593"
    Private Const IID_IMFMediaEventGenerator As String = "2cd0bd52-bcd5-4b89-b62c-eadc0c031e7d"
    Private Const IID_IMFPresentationDescriptor As String = "03cb2711-24d7-4db6-a17f-f3a7a479a536"
#End Region

#Region "Enum"
    Private Enum MF_OBJECT_TYPE As Integer
        MF_OBJECT_MEDIASOURCE = &H0
        MF_OBJECT_BYTESTREAM = &H1
        MF_OBJECT_INVALID = &H2
    End Enum

    Private Enum MF_TOPOLOGY_TYPE As Integer
        MF_TOPOLOGY_OUTPUT_NODE = &H0
        MF_TOPOLOGY_SOURCESTREAM_NODE = &H1
        MF_TOPOLOGY_TRANSFORM_NODE = &H2
        MF_TOPOLOGY_TEE_NODE = &H3
    End Enum

    Private Enum MFCLOCK_STATE As Integer
        MFCLOCK_STATE_INVALID = &H0
        MFCLOCK_STATE_RUNNING = &H1
        MFCLOCK_STATE_STOPPED = &H2
        MFCLOCK_STATE_PAUSED = &H3
    End Enum

    Private Enum MF_RESOLUTION_TYPE As Integer
        MF_RESOLUTION_MEDIASOURCE = &H1
        MF_RESOLUTION_BYTESTREAM = &H2
        MF_RESOLUTION_CONTENT_DOES_NOT_HAVE_TO_MATCH_EXTENSION_OR_MIME_TYPE = &H10
        MF_RESOLUTION_KEEP_BYTE_STREAM_ALIVE_ON_FAIL = &H20
        MF_RESOLUTION_DISABLE_LOCAL_PLUGINS = &H40
        MF_RESOLUTION_PLUGIN_CONTROL_POLICY_APPROVED_ONLY = &H80
        MF_RESOLUTION_PLUGIN_CONTROL_POLICY_WEB_ONLY = &H100
        MF_RESOLUTION_PLUGIN_CONTROL_POLICY_WEB_ONLY_EDGEMODE = &H200
        MF_RESOLUTION_ENABLE_STORE_PLUGINS = &H400
        MF_RESOLUTION_READ = &H10000
        MF_RESOLUTION_WRITE = &H20000
    End Enum

    Private Enum MF_ATTRIBUTE_TYPE As Integer
        MF_ATTRIBUTE_UINT32 = VARTYPE.VT_UI4
        MF_ATTRIBUTE_UINT64 = VARTYPE.VT_UI8
        MF_ATTRIBUTE_DOUBLE = VARTYPE.VT_R8
        MF_ATTRIBUTE_GUID = VARTYPE.VT_CLSID
        MF_ATTRIBUTE_STRING = VARTYPE.VT_LPWSTR
        MF_ATTRIBUTE_BLOB = VARTYPE.VT_VECTOR Or VARTYPE.VT_UI1
        MF_ATTRIBUTE_IUNKNOWN = VARTYPE.VT_UNKNOWN
    End Enum

    Private Enum MF_ATTRIBUTES_MATCH_TYPE As Integer
        MF_ATTRIBUTES_MATCH_OUR_ITEMS = 0
        MF_ATTRIBUTES_MATCH_THEIR_ITEMS = 1
        MF_ATTRIBUTES_MATCH_ALL_ITEMS = 2
        MF_ATTRIBUTES_MATCH_INTERSECTION = 3
        MF_ATTRIBUTES_MATCH_SMALLER = 4
    End Enum
    Private Enum VARTYPE As Short
        VT_EMPTY = 0
        VT_NULL = 1
        VT_I2 = 2
        VT_I4 = 3
        VT_R4 = 4
        VT_R8 = 5
        VT_CY = 6
        VT_DATE = 7
        VT_BSTR = 8
        VT_DISPATCH = 9
        VT_ERROR = 10
        VT_BOOL = 11
        VT_VARIANT = 12
        VT_UNKNOWN = 13
        VT_DECIMAL = 14
        VT_I1 = 16
        VT_UI1 = 17
        VT_UI2 = 18
        VT_UI4 = 19
        VT_I8 = 20
        VT_UI8 = 21
        VT_INT = 22
        VT_UINT = 23
        VT_VOID = 24
        VT_HRESULT = 25
        VT_PTR = 26
        VT_SAFEARRAY = 27
        VT_CARRAY = 28
        VT_USERDEFINED = 29
        VT_LPSTR = 30
        VT_LPWSTR = 31
        VT_RECORD = 36
        VT_FILETIME = 64
        VT_BLOB = 65
        VT_STREAM = 66
        VT_STORAGE = 67
        VT_STREAMED_OBJECT = 68
        VT_STORED_OBJECT = 69
        VT_BLOB_OBJECT = 70
        VT_CF = 71
        VT_CLSID = 72
        VT_ILLEGALMASKED = 4095
        VT_VECTOR = 4096
        VT_ARRAY = 8192
        VT_BYREF = 16384
    End Enum
#End Region

#Region "Structure"

    Private Structure CY
        Dim Lo As UInteger
        Dim Hi As Integer
    End Structure

    Private Structure CArray
        Dim cElems As UInteger
        Dim pElems As IntPtr
    End Structure

    Private Structure BLOB
        Dim cbSize As UInteger
        Dim pBlobData As IntPtr
    End Structure

    Private Structure BSTRBLOB
        Dim cbSize As UInteger
        Dim pData As IntPtr
    End Structure

    Private Structure PropVariantUnion
        <FieldOffset(0)> Dim cVal As SByte
        <FieldOffset(0)> Dim bVal As Byte
        <FieldOffset(0)> Dim iVal As Short
        <FieldOffset(0)> Dim uiVal As UShort
        <FieldOffset(0)> Dim lVal As Integer
        <FieldOffset(0)> Dim ulVal As UInteger
        <FieldOffset(0)> Dim intVal As Integer
        <FieldOffset(0)> Dim uintVal As UInteger
        <FieldOffset(0)> Dim hVal As Long
        <FieldOffset(0)> Dim uhVal As ULong
        <FieldOffset(0)> Dim fltVal As Single
        <FieldOffset(0)> Dim dblVal As Double
        <FieldOffset(0)> Dim boolVal As Short
        <FieldOffset(0)> Dim scode As Integer
        <FieldOffset(0)> Dim cyVal As CY
        <FieldOffset(0)> Dim [date] As Double
        <FieldOffset(0)> Dim filetime As ComTypes.FILETIME
        <FieldOffset(0)> Dim puuid As IntPtr
        <FieldOffset(0)> Dim pclipdata As IntPtr
        <FieldOffset(0)> Dim bstrVal As IntPtr
        <FieldOffset(0)> Dim bstrblobVal As BSTRBLOB
        <FieldOffset(0)> Dim blob As BLOB
        <FieldOffset(0)> Dim pszVal As IntPtr
        <FieldOffset(0)> Dim pwszVal As IntPtr
        <FieldOffset(0)> Dim punkVal As IntPtr
        <FieldOffset(0)> Dim pdispVal As IntPtr
        <FieldOffset(0)> Dim pStream As IntPtr
        <FieldOffset(0)> Dim pStorage As IntPtr
        <FieldOffset(0)> Dim pVersionedStream As IntPtr
        <FieldOffset(0)> Dim parray As IntPtr
        <FieldOffset(0)> Dim cArray As CArray
        <FieldOffset(0)> Dim pcVal As IntPtr
        <FieldOffset(0)> Dim pbVal As IntPtr
        <FieldOffset(0)> Dim piVal As IntPtr
        <FieldOffset(0)> Dim puiVal As IntPtr
        <FieldOffset(0)> Dim plVal As IntPtr
        <FieldOffset(0)> Dim pulVal As IntPtr
        <FieldOffset(0)> Dim pintVal As IntPtr
        <FieldOffset(0)> Dim puintVal As IntPtr
        <FieldOffset(0)> Dim pfltVal As IntPtr
        <FieldOffset(0)> Dim pdblVal As IntPtr
        <FieldOffset(0)> Dim pboolVal As IntPtr
        <FieldOffset(0)> Dim pdecVal As IntPtr
        <FieldOffset(0)> Dim pscode As IntPtr
        <FieldOffset(0)> Dim pcyVal As IntPtr
        <FieldOffset(0)> Dim pdate As IntPtr
        <FieldOffset(0)> Dim pbstrVal As IntPtr
        <FieldOffset(0)> Dim ppunkVal As IntPtr
        <FieldOffset(0)> Dim ppdispVal As IntPtr
        <FieldOffset(0)> Dim pparray As IntPtr
        <FieldOffset(0)> Dim pvarVal As IntPtr
    End Structure

    Private Structure PROPVARIANT
        <FieldOffset(0)> Dim vt As VARTYPE
        <FieldOffset(2)> Dim wReserved1 As UShort
        <FieldOffset(4)> Dim wReserved2 As UShort
        <FieldOffset(6)> Dim wReserved3 As UShort
        <FieldOffset(8)> Dim union As PropVariantUnion
    End Structure
#End Region

#Region "Interface IMFClock"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IMFClock)>
    Private Interface IMFClock

        <PreserveSig> Function GetClockCharacteristics() As Integer ' nur Dummyfunktion

        <PreserveSig> Function GetCorrelatedTime(<[In]> ByVal dwReserved As Integer,
                                                 <Out> ByRef pllClockTime As ULong,
                                                 <Out> ByRef phnsSystemTime As ULong) As Integer

        <PreserveSig> Function GetContinuityKey() As Integer ' nur Dummyfunktion

        <PreserveSig> Function GetState(<[In]> ByVal dwReserved As Integer,
                                        <Out> ByRef peClockState As MFCLOCK_STATE) As Integer

        <PreserveSig> Function GetProperties() As Integer ' nur Dummyfunktion

    End Interface
#End Region

#Region "Interface IMFActivate"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IMFActivate)>
    Private Interface IMFActivate
        Inherits IMFAttributes

        <PreserveSig> Shadows Function GetItem(<[In]> ByRef guidKey As Guid,
                                               <Out> ByRef pValue As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function GetItemType(<[In]> ByRef guidKey As Guid,
                                                   <Out> ByRef pType As MF_ATTRIBUTE_TYPE) As Integer

        <PreserveSig> Shadows Function CompareItem(<[In]> ByRef guidKey As Guid,
                                                   <[In]> ByRef pValue As PROPVARIANT,
                                                   <Out, MarshalAs(UnmanagedType.Bool)> ByRef pbResult As Boolean) As Integer

        <PreserveSig> Shadows Function Compare(<[In], MarshalAs(UnmanagedType.Interface)> ByVal pTheirs As IMFAttributes,
                                               <[In]> ByVal MatchType As MF_ATTRIBUTES_MATCH_TYPE,
                                               <Out, MarshalAs(UnmanagedType.Bool)> ByRef pbResult As Boolean) As Integer

        <PreserveSig> Shadows Function GetUInt32(<[In]> ByRef guidKey As Guid,
                                                 <Out> ByRef punValue As UInteger) As Integer

        <PreserveSig> Shadows Function GetUINT64(<[In]> ByRef guidKey As Guid,
                                                 <Out> ByRef punValue As ULong) As Integer

        <PreserveSig> Shadows Function GetDouble(<[In]> ByRef guidKey As Guid,
                                                 <Out> ByRef punValue As Double) As Integer

        <PreserveSig> Shadows Function GetGUID(<[In]> ByRef guidKey As Guid,
                                               <Out> ByRef pguidValue As Guid) As Integer

        <PreserveSig> Shadows Function GetStringLength(<[In]> ByRef guidKey As Guid,
                                                       <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetString(<[In]> ByRef guidKey As Guid,
                                                 <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef pwszValue As String,
                                                 <[In]> ByVal cchBufSize As UInteger,
                                                 <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetAllocatedString(<[In]> ByRef guidKey As Guid,
                                                          <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef ppwszValue As String,
                                                          <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetBlobSize(<[In]> ByRef guidKey As Guid,
                                           <Out> ByRef pcbBlobSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetBlob(<[In]> ByRef guidKey As Guid,
                                       <Out, MarshalAs(UnmanagedType.LPArray)> ByRef pBuf As Byte(),
                                       <[In]> ByVal cbBufSize As UInteger,
                                       <Out> ByRef pcbBlobSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetAllocatedBlob(<[In]> ByRef guidKey As Guid,
                                                <Out, MarshalAs(UnmanagedType.LPArray)> ByRef pBuf As Byte(),
                                                <Out> ByRef pcbSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetUnknown(<[In]> ByRef guidKey As Guid,
                                          <[In]> ByRef riid As Guid,
                                          <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppv As Object) As Integer

        <PreserveSig> Shadows Function SetItem(<[In]> ByRef guidKey As Guid,
                                       <[In]> ByRef Value As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function DeleteItem(<[In]> ByRef guidKey As Guid) As Integer

        <PreserveSig> Shadows Function DeleteAllItems() As Integer

        <PreserveSig> Shadows Function SetUInt32(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal unValue As UInteger) As Integer

        <PreserveSig> Shadows Function SetUINT64(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal unValue As ULong) As Integer

        <PreserveSig> Shadows Function SetDouble(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal fValue As Double) As Integer

        <PreserveSig> Shadows Function SetGUID(<[In]> ByRef guidKey As Guid,
                                       <[In]> ByRef guidValue As Guid) As Integer

        <PreserveSig> Shadows Function SetString(<[In]> ByRef guidKey As Guid,
                                         <[In], MarshalAs(UnmanagedType.LPWStr)> ByVal wszValue As String) As Integer

        <PreserveSig> Shadows Function SetBlob(<[In]> ByRef guidKey As Guid,
                                       <[In], MarshalAs(UnmanagedType.LPArray)> ByVal pBuf As Byte(),
                                       <[In]> ByVal cbBufSize As UInteger) As Integer

        <PreserveSig> Shadows Function SetUnknown(<[In]> ByRef guidKey As Guid,
                                          <[In], MarshalAs(UnmanagedType.Interface)> ByVal pUnknown As Object) As Integer

        <PreserveSig> Shadows Function LockStore() As Integer

        <PreserveSig> Shadows Function UnlockStore() As Integer

        <PreserveSig> Shadows Function GetCount(<Out> ByRef pcItems As UInteger) As Integer

        <PreserveSig> Shadows Function GetItemByIndex(<[In]> ByVal unIndex As UInteger,
                                              <[In]> ByRef pguidKey As Guid,
                                              <Out> ByRef pValue As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function CopyAllItems(<Out, MarshalAs(UnmanagedType.Interface)> ByRef pDest As IMFAttributes) As Integer


        <PreserveSig> Function ActivateObject(<[In]> ByRef riid As Guid, <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppv As Object) As Integer

        <PreserveSig> Function ShutdownObject() As Integer

        <PreserveSig> Function DetachObject() As Integer

    End Interface
#End Region

#Region "Interface IMFTopology"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IMFTopology)>
    Private Interface IMFTopology
        Inherits IMFAttributes

        <PreserveSig> Shadows Function GetItem(<[In]> ByRef guidKey As Guid,
                                       <Out> ByRef pValue As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function GetItemType(<[In]> ByRef guidKey As Guid,
                                           <Out> ByRef pType As MF_ATTRIBUTE_TYPE) As Integer

        <PreserveSig> Shadows Function CompareItem(<[In]> ByRef guidKey As Guid,
                                           <[In]> ByRef pValue As PROPVARIANT,
                                           <Out, MarshalAs(UnmanagedType.Bool)> ByRef pbResult As Boolean) As Integer

        <PreserveSig> Shadows Function Compare(<[In], MarshalAs(UnmanagedType.Interface)> ByVal pTheirs As IMFAttributes,
                                       <[In]> ByVal MatchType As MF_ATTRIBUTES_MATCH_TYPE,
                                       <Out, MarshalAs(UnmanagedType.Bool)> ByRef pbResult As Boolean) As Integer

        <PreserveSig> Shadows Function GetUInt32(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As UInteger) As Integer

        <PreserveSig> Shadows Function GetUINT64(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As ULong) As Integer

        <PreserveSig> Shadows Function GetDouble(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As Double) As Integer

        <PreserveSig> Shadows Function GetGUID(<[In]> ByRef guidKey As Guid,
                                       <Out> ByRef pguidValue As Guid) As Integer

        <PreserveSig> Shadows Function GetStringLength(<[In]> ByRef guidKey As Guid,
                                               <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetString(<[In]> ByRef guidKey As Guid,
                                         <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef pwszValue As String,
                                         <[In]> ByVal cchBufSize As UInteger,
                                         <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetAllocatedString(<[In]> ByRef guidKey As Guid,
                                                  <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef ppwszValue As String,
                                                  <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetBlobSize(<[In]> ByRef guidKey As Guid,
                                           <Out> ByRef pcbBlobSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetBlob(<[In]> ByRef guidKey As Guid,
                                       <Out, MarshalAs(UnmanagedType.LPArray)> ByRef pBuf As Byte(),
                                       <[In]> ByVal cbBufSize As UInteger,
                                       <Out> ByRef pcbBlobSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetAllocatedBlob(<[In]> ByRef guidKey As Guid,
                                                <Out, MarshalAs(UnmanagedType.LPArray)> ByRef pBuf As Byte(),
                                                <Out> ByRef pcbSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetUnknown(<[In]> ByRef guidKey As Guid,
                                          <[In]> ByRef riid As Guid,
                                          <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppv As Object) As Integer

        <PreserveSig> Shadows Function SetItem(<[In]> ByRef guidKey As Guid,
                                       <[In]> ByRef Value As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function DeleteItem(<[In]> ByRef guidKey As Guid) As Integer

        <PreserveSig> Shadows Function DeleteAllItems() As Integer

        <PreserveSig> Shadows Function SetUInt32(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal unValue As UInteger) As Integer

        <PreserveSig> Shadows Function SetUINT64(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal unValue As ULong) As Integer

        <PreserveSig> Shadows Function SetDouble(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal fValue As Double) As Integer

        <PreserveSig> Shadows Function SetGUID(<[In]> ByRef guidKey As Guid,
                                       <[In]> ByRef guidValue As Guid) As Integer

        <PreserveSig> Shadows Function SetString(<[In]> ByRef guidKey As Guid,
                                         <[In], MarshalAs(UnmanagedType.LPWStr)> ByVal wszValue As String) As Integer

        <PreserveSig> Shadows Function SetBlob(<[In]> ByRef guidKey As Guid,
                                       <[In], MarshalAs(UnmanagedType.LPArray)> ByVal pBuf As Byte(),
                                       <[In]> ByVal cbBufSize As UInteger) As Integer

        <PreserveSig> Shadows Function SetUnknown(<[In]> ByRef guidKey As Guid,
                                          <[In], MarshalAs(UnmanagedType.Interface)> ByVal pUnknown As Object) As Integer

        <PreserveSig> Shadows Function LockStore() As Integer

        <PreserveSig> Shadows Function UnlockStore() As Integer

        <PreserveSig> Shadows Function GetCount(<Out> ByRef pcItems As UInteger) As Integer

        <PreserveSig> Shadows Function GetItemByIndex(<[In]> ByVal unIndex As UInteger,
                                              <[In]> ByRef pguidKey As Guid,
                                              <Out> ByRef pValue As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function CopyAllItems(<Out, MarshalAs(UnmanagedType.Interface)> ByRef pDest As IMFAttributes) As Integer


        <PreserveSig> Function GetTopologyID() As Integer ' nur Dummyfunktion

        <PreserveSig> Function AddNode(<[In], MarshalAs(UnmanagedType.Interface)> ByVal pNode As IMFTopologyNode) As Integer

        'weitere Funktionen dieses Interfaces
        'RemoveNode
        'GetNodeCount
        'GetNode
        'Clear
        'CloneFrom
        'GetNodeByID
        'GetSourceNodeCollection
        'GetOutputNodeCollection

    End Interface
#End Region

#Region "Interface IMFAttributes"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IMFAttributes)>
    Private Interface IMFAttributes

        <PreserveSig> Function GetItem(<[In]> ByRef guidKey As Guid,
                                       <Out> ByRef pValue As PROPVARIANT) As Integer

        <PreserveSig> Function GetItemType(<[In]> ByRef guidKey As Guid,
                                           <Out> ByRef pType As MF_ATTRIBUTE_TYPE) As Integer

        <PreserveSig> Function CompareItem(<[In]> ByRef guidKey As Guid,
                                           <[In]> ByRef pValue As PROPVARIANT,
                                           <Out, MarshalAs(UnmanagedType.Bool)> ByRef pbResult As Boolean) As Integer

        <PreserveSig> Function Compare(<[In], MarshalAs(UnmanagedType.Interface)> ByVal pTheirs As IMFAttributes,
                                       <[In]> ByVal MatchType As MF_ATTRIBUTES_MATCH_TYPE,
                                       <Out, MarshalAs(UnmanagedType.Bool)> ByRef pbResult As Boolean) As Integer

        <PreserveSig> Function GetUInt32(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As UInteger) As Integer

        <PreserveSig> Function GetUINT64(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As ULong) As Integer

        <PreserveSig> Function GetDouble(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As Double) As Integer

        <PreserveSig> Function GetGUID(<[In]> ByRef guidKey As Guid,
                                       <Out> ByRef pguidValue As Guid) As Integer

        <PreserveSig> Function GetStringLength(<[In]> ByRef guidKey As Guid,
                                               <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Function GetString(<[In]> ByRef guidKey As Guid,
                                         <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef pwszValue As String,
                                         <[In]> ByVal cchBufSize As UInteger,
                                         <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Function GetAllocatedString(<[In]> ByRef guidKey As Guid,
                                                  <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef ppwszValue As String,
                                                  <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Function GetBlobSize(<[In]> ByRef guidKey As Guid,
                                           <Out> ByRef pcbBlobSize As UInteger) As Integer

        <PreserveSig> Function GetBlob(<[In]> ByRef guidKey As Guid,
                                       <Out, MarshalAs(UnmanagedType.LPArray)> ByRef pBuf As Byte(),
                                       <[In]> ByVal cbBufSize As UInteger,
                                       <Out> ByRef pcbBlobSize As UInteger) As Integer

        <PreserveSig> Function GetAllocatedBlob(<[In]> ByRef guidKey As Guid,
                                                <Out, MarshalAs(UnmanagedType.LPArray)> ByRef pBuf As Byte(),
                                                <Out> ByRef pcbSize As UInteger) As Integer

        <PreserveSig> Function GetUnknown(<[In]> ByRef guidKey As Guid,
                                          <[In]> ByRef riid As Guid,
                                          <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppv As Object) As Integer

        <PreserveSig> Function SetItem(<[In]> ByRef guidKey As Guid,
                                       <[In]> ByRef Value As PROPVARIANT) As Integer

        <PreserveSig> Function DeleteItem(<[In]> ByRef guidKey As Guid) As Integer

        <PreserveSig> Function DeleteAllItems() As Integer

        <PreserveSig> Function SetUInt32(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal unValue As UInteger) As Integer

        <PreserveSig> Function SetUINT64(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal unValue As ULong) As Integer

        <PreserveSig> Function SetDouble(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal fValue As Double) As Integer

        <PreserveSig> Function SetGUID(<[In]> ByRef guidKey As Guid,
                                       <[In]> ByRef guidValue As Guid) As Integer

        <PreserveSig> Function SetString(<[In]> ByRef guidKey As Guid,
                                         <[In], MarshalAs(UnmanagedType.LPWStr)> ByVal wszValue As String) As Integer

        <PreserveSig> Function SetBlob(<[In]> ByRef guidKey As Guid,
                                       <[In], MarshalAs(UnmanagedType.LPArray)> ByVal pBuf As Byte(),
                                       <[In]> ByVal cbBufSize As UInteger) As Integer

        <PreserveSig> Function SetUnknown(<[In]> ByRef guidKey As Guid,
                                          <[In], MarshalAs(UnmanagedType.Interface)> ByVal pUnknown As Object) As Integer

        <PreserveSig> Function LockStore() As Integer

        <PreserveSig> Function UnlockStore() As Integer

        <PreserveSig> Function GetCount(<Out> ByRef pcItems As UInteger) As Integer

        <PreserveSig> Function GetItemByIndex(<[In]> ByVal unIndex As UInteger,
                                              <[In]> ByRef pguidKey As Guid,
                                              <Out> ByRef pValue As PROPVARIANT) As Integer

        <PreserveSig> Function CopyAllItems(<Out, MarshalAs(UnmanagedType.Interface)> ByRef pDest As IMFAttributes) As Integer

    End Interface
#End Region

#Region "Interface IMFMediaSource"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IMFMediaSource)>
    Private Interface IMFMediaSource
        Inherits IMFMediaEventGenerator

        ' nur Dummyfunktionen von IMFMediaEventGenerator
        <PreserveSig> Shadows Function GetEvent() As Integer
        <PreserveSig> Shadows Function BeginGetEvent() As Integer
        <PreserveSig> Shadows Function EndGetEvent() As Integer
        <PreserveSig> Shadows Function QueueEvent() As Integer


        <PreserveSig> Function GetCharacteristics() As Integer ' nur Dummyfunktion

        <PreserveSig> Function CreatePresentationDescriptor(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppPresentationDescriptor As IMFPresentationDescriptor) As Integer

        <PreserveSig> Function Start() As Integer ' nur Dummyfunktion

        <PreserveSig> Function [Stop]() As Integer ' nur Dummyfunktion

        <PreserveSig> Function Pause() As Integer ' nur Dummyfunktion

        <PreserveSig> Function Shutdown() As Integer

    End Interface
#End Region

#Region "Interface IMFMediaSession"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IMFMediaSession)>
    Private Interface IMFMediaSession
        Inherits IMFMediaEventGenerator

        ' nur Dummyfunktionen von IMFMediaEventGenerator
        <PreserveSig> Shadows Function GetEvent() As Integer
        <PreserveSig> Shadows Function BeginGetEvent() As Integer
        <PreserveSig> Shadows Function EndGetEvent() As Integer
        <PreserveSig> Shadows Function QueueEvent() As Integer


        <PreserveSig> Function SetTopology(<[In]> ByVal dwSetTopologyFlags As Integer, <[In], MarshalAs(UnmanagedType.Interface)> ByVal pTopology As IMFTopology) As Integer

        <PreserveSig> Function ClearTopologies() As Integer

        <PreserveSig> Function Start(<[In]> ByRef pguidTimeFormat As Guid, <[In]> ByRef pvarStartPosition As PROPVARIANT) As Integer

        <PreserveSig> Function Pause() As Integer

        <PreserveSig> Function [Stop]() As Integer

        <PreserveSig> Function Close() As Integer

        <PreserveSig> Function Shutdown() As Integer

        <PreserveSig> Function GetClock(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppClock As IMFClock) As Integer

        'weitere Funktionen dieses Interfaces
        'GetSessionCapabilities
        'GetFullTopology

    End Interface
#End Region

#Region "Interface IMFTopologyNode"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IMFTopologyNode)>
    Private Interface IMFTopologyNode
        Inherits IMFAttributes

        <PreserveSig> Shadows Function GetItem(<[In]> ByRef guidKey As Guid,
                                       <Out> ByRef pValue As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function GetItemType(<[In]> ByRef guidKey As Guid,
                                           <Out> ByRef pType As MF_ATTRIBUTE_TYPE) As Integer

        <PreserveSig> Shadows Function CompareItem(<[In]> ByRef guidKey As Guid,
                                           <[In]> ByRef pValue As PROPVARIANT,
                                           <Out, MarshalAs(UnmanagedType.Bool)> ByRef pbResult As Boolean) As Integer

        <PreserveSig> Shadows Function Compare(<[In], MarshalAs(UnmanagedType.Interface)> ByVal pTheirs As IMFAttributes,
                                       <[In]> ByVal MatchType As MF_ATTRIBUTES_MATCH_TYPE,
                                       <Out, MarshalAs(UnmanagedType.Bool)> ByRef pbResult As Boolean) As Integer

        <PreserveSig> Shadows Function GetUInt32(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As UInteger) As Integer

        <PreserveSig> Shadows Function GetUINT64(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As ULong) As Integer

        <PreserveSig> Shadows Function GetDouble(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As Double) As Integer

        <PreserveSig> Shadows Function GetGUID(<[In]> ByRef guidKey As Guid,
                                       <Out> ByRef pguidValue As Guid) As Integer

        <PreserveSig> Shadows Function GetStringLength(<[In]> ByRef guidKey As Guid,
                                               <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetString(<[In]> ByRef guidKey As Guid,
                                         <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef pwszValue As String,
                                         <[In]> ByVal cchBufSize As UInteger,
                                         <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetAllocatedString(<[In]> ByRef guidKey As Guid,
                                                  <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef ppwszValue As String,
                                                  <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetBlobSize(<[In]> ByRef guidKey As Guid,
                                           <Out> ByRef pcbBlobSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetBlob(<[In]> ByRef guidKey As Guid,
                                       <Out, MarshalAs(UnmanagedType.LPArray)> ByRef pBuf As Byte(),
                                       <[In]> ByVal cbBufSize As UInteger,
                                       <Out> ByRef pcbBlobSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetAllocatedBlob(<[In]> ByRef guidKey As Guid,
                                                <Out, MarshalAs(UnmanagedType.LPArray)> ByRef pBuf As Byte(),
                                                <Out> ByRef pcbSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetUnknown(<[In]> ByRef guidKey As Guid,
                                          <[In]> ByRef riid As Guid,
                                          <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppv As Object) As Integer

        <PreserveSig> Shadows Function SetItem(<[In]> ByRef guidKey As Guid,
                                       <[In]> ByRef Value As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function DeleteItem(<[In]> ByRef guidKey As Guid) As Integer

        <PreserveSig> Shadows Function DeleteAllItems() As Integer

        <PreserveSig> Shadows Function SetUInt32(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal unValue As UInteger) As Integer

        <PreserveSig> Shadows Function SetUINT64(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal unValue As ULong) As Integer

        <PreserveSig> Shadows Function SetDouble(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal fValue As Double) As Integer

        <PreserveSig> Shadows Function SetGUID(<[In]> ByRef guidKey As Guid,
                                       <[In]> ByRef guidValue As Guid) As Integer

        <PreserveSig> Shadows Function SetString(<[In]> ByRef guidKey As Guid,
                                         <[In], MarshalAs(UnmanagedType.LPWStr)> ByVal wszValue As String) As Integer

        <PreserveSig> Shadows Function SetBlob(<[In]> ByRef guidKey As Guid,
                                       <[In], MarshalAs(UnmanagedType.LPArray)> ByVal pBuf As Byte(),
                                       <[In]> ByVal cbBufSize As UInteger) As Integer

        <PreserveSig> Shadows Function SetUnknown(<[In]> ByRef guidKey As Guid,
                                          <[In], MarshalAs(UnmanagedType.Interface)> ByVal pUnknown As Object) As Integer

        <PreserveSig> Shadows Function LockStore() As Integer

        <PreserveSig> Shadows Function UnlockStore() As Integer

        <PreserveSig> Shadows Function GetCount(<Out> ByRef pcItems As UInteger) As Integer

        <PreserveSig> Shadows Function GetItemByIndex(<[In]> ByVal unIndex As UInteger,
                                              <[In]> ByRef pguidKey As Guid,
                                              <Out> ByRef pValue As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function CopyAllItems(<Out, MarshalAs(UnmanagedType.Interface)> ByRef pDest As IMFAttributes) As Integer


        <PreserveSig> Function SetObject(<[In], MarshalAs(UnmanagedType.Interface)> ByVal pObject As Object) As Integer

        <PreserveSig> Function GetObject() As Integer ' nur Dummyfunktion

        <PreserveSig> Function GetNodeType() As Integer ' nur Dummyfunktion

        <PreserveSig> Function GetTopoNodeID() As Integer ' nur Dummyfunktion

        <PreserveSig> Function SetTopoNodeID() As Integer ' nur Dummyfunktion

        <PreserveSig> Function GetInputCount() As Integer ' nur Dummyfunktion

        <PreserveSig> Function GetOutputCount() As Integer ' nur Dummyfunktion

        <PreserveSig> Function ConnectOutput(<[In]> ByVal dwOutputIndex As Integer,
                                             <[In], MarshalAs(UnmanagedType.Interface)> ByVal pDownstreamNode As IMFTopologyNode,
                                             <[In]> ByVal dwInputIndexOnDownstreamNode As Integer) As Integer

        'weitere Funktionen dieses Interfaces
        'DisconnectOutput
        'GetInput
        'GetOutput
        'SetOutputPrefType
        'GetOutputPrefType
        'SetInputPrefType
        'GetInputPrefType
        'CloneFrom

    End Interface
#End Region

#Region "Interface IMFSourceResolver"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IMFSourceResolver)>
    Private Interface IMFSourceResolver

        <PreserveSig> Function CreateObjectFromURL(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pwszURL As String,
                                                   <[In]> ByVal dwFlags As MF_RESOLUTION_TYPE,
                                                   <[In]> ByVal pProps As IntPtr,
                                                   <Out> ByRef pObjectType As MF_OBJECT_TYPE,
                                                   <Out> ByRef ppObject As IntPtr) As Integer


        'weitere Funktionen dieses Interfaces
        'CreateObjectFromURL
        'CreateObjectFromByteStream
        'BeginCreateObjectFromURL
        'EndCreateObjectFromURL
        'BeginCreateObjectFromByteStream
        'EndCreateObjectFromByteStream
        'CancelObjectCreation

    End Interface
#End Region

#Region "Interface IMFMediaTypeHandler"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IMFMediaTypeHandler)>
    Private Interface IMFMediaTypeHandler
        <PreserveSig> Function IsMediaTypeSupported() As Integer ' nur Dummyfunktion

        <PreserveSig> Function GetMediaTypeCount() As Integer ' nur Dummyfunktion

        <PreserveSig> Function GetMediaTypeByIndex() As Integer ' nur Dummyfunktion

        <PreserveSig> Function SetCurrentMediaType() As Integer ' nur Dummyfunktion

        <PreserveSig> Function GetCurrentMediaType() As Integer ' nur Dummyfunktion

        <PreserveSig> Function GetMajorType(<Out> ByRef pguidMajorType As Guid) As Integer
    End Interface
#End Region

#Region "Interface IMFStreamDescriptor"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IMFStreamDescriptor)>
    Private Interface IMFStreamDescriptor
        Inherits IMFAttributes

        <PreserveSig> Shadows Function GetItem(<[In]> ByRef guidKey As Guid,
                                       <Out> ByRef pValue As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function GetItemType(<[In]> ByRef guidKey As Guid,
                                           <Out> ByRef pType As MF_ATTRIBUTE_TYPE) As Integer

        <PreserveSig> Shadows Function CompareItem(<[In]> ByRef guidKey As Guid,
                                           <[In]> ByRef pValue As PROPVARIANT,
                                           <Out, MarshalAs(UnmanagedType.Bool)> ByRef pbResult As Boolean) As Integer

        <PreserveSig> Shadows Function Compare(<[In], MarshalAs(UnmanagedType.Interface)> ByVal pTheirs As IMFAttributes,
                                       <[In]> ByVal MatchType As MF_ATTRIBUTES_MATCH_TYPE,
                                       <Out, MarshalAs(UnmanagedType.Bool)> ByRef pbResult As Boolean) As Integer

        <PreserveSig> Shadows Function GetUInt32(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As UInteger) As Integer

        <PreserveSig> Shadows Function GetUINT64(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As ULong) As Integer

        <PreserveSig> Shadows Function GetDouble(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As Double) As Integer

        <PreserveSig> Shadows Function GetGUID(<[In]> ByRef guidKey As Guid,
                                       <Out> ByRef pguidValue As Guid) As Integer

        <PreserveSig> Shadows Function GetStringLength(<[In]> ByRef guidKey As Guid,
                                               <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetString(<[In]> ByRef guidKey As Guid,
                                         <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef pwszValue As String,
                                         <[In]> ByVal cchBufSize As UInteger,
                                         <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetAllocatedString(<[In]> ByRef guidKey As Guid,
                                                  <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef ppwszValue As String,
                                                  <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetBlobSize(<[In]> ByRef guidKey As Guid,
                                           <Out> ByRef pcbBlobSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetBlob(<[In]> ByRef guidKey As Guid,
                                       <Out, MarshalAs(UnmanagedType.LPArray)> ByRef pBuf As Byte(),
                                       <[In]> ByVal cbBufSize As UInteger,
                                       <Out> ByRef pcbBlobSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetAllocatedBlob(<[In]> ByRef guidKey As Guid,
                                                <Out, MarshalAs(UnmanagedType.LPArray)> ByRef pBuf As Byte(),
                                                <Out> ByRef pcbSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetUnknown(<[In]> ByRef guidKey As Guid,
                                          <[In]> ByRef riid As Guid,
                                          <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppv As Object) As Integer

        <PreserveSig> Shadows Function SetItem(<[In]> ByRef guidKey As Guid,
                                       <[In]> ByRef Value As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function DeleteItem(<[In]> ByRef guidKey As Guid) As Integer

        <PreserveSig> Shadows Function DeleteAllItems() As Integer

        <PreserveSig> Shadows Function SetUInt32(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal unValue As UInteger) As Integer

        <PreserveSig> Shadows Function SetUINT64(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal unValue As ULong) As Integer

        <PreserveSig> Shadows Function SetDouble(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal fValue As Double) As Integer

        <PreserveSig> Shadows Function SetGUID(<[In]> ByRef guidKey As Guid,
                                       <[In]> ByRef guidValue As Guid) As Integer

        <PreserveSig> Shadows Function SetString(<[In]> ByRef guidKey As Guid,
                                         <[In], MarshalAs(UnmanagedType.LPWStr)> ByVal wszValue As String) As Integer

        <PreserveSig> Shadows Function SetBlob(<[In]> ByRef guidKey As Guid,
                                       <[In], MarshalAs(UnmanagedType.LPArray)> ByVal pBuf As Byte(),
                                       <[In]> ByVal cbBufSize As UInteger) As Integer

        <PreserveSig> Shadows Function SetUnknown(<[In]> ByRef guidKey As Guid,
                                          <[In], MarshalAs(UnmanagedType.Interface)> ByVal pUnknown As Object) As Integer

        <PreserveSig> Shadows Function LockStore() As Integer

        <PreserveSig> Shadows Function UnlockStore() As Integer

        <PreserveSig> Shadows Function GetCount(<Out> ByRef pcItems As UInteger) As Integer

        <PreserveSig> Shadows Function GetItemByIndex(<[In]> ByVal unIndex As UInteger,
                                              <[In]> ByRef pguidKey As Guid,
                                              <Out> ByRef pValue As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function CopyAllItems(<Out, MarshalAs(UnmanagedType.Interface)> ByRef pDest As IMFAttributes) As Integer


        <PreserveSig> Function GetStreamIdentifier(<Out> ByRef pdwStreamIdentifier As Integer) As Integer

        <PreserveSig> Function GetMediaTypeHandler(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppMediaTypeHandle As IMFMediaTypeHandler) As Integer

    End Interface
#End Region

#Region "Interface IMFAudioStreamVolume"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IMFAudioStreamVolume)>
    Private Interface IMFAudioStreamVolume

        <PreserveSig> Function GetChannelCount(<Out> ByRef pdwCount As Integer) As Integer

        <PreserveSig> Function SetChannelVolume(<[In]> ByVal dwIndex As Integer, <[In]> ByVal fLevel As Single) As Integer

        <PreserveSig> Function GetChannelVolume(<[In]> ByVal dwIndex As Integer, <Out> ByRef pfLevel As Single) As Integer

        ' weitere Funktionen dieses Interfaces
        'SetAllVolumes
        'GetAllVolumes

    End Interface
#End Region

#Region "Interface IMFMediaEventGenerator"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IMFMediaEventGenerator)>
    Private Interface IMFMediaEventGenerator
        ' nur Dummyfunktionen
        <PreserveSig> Function GetEvent() As Integer
        <PreserveSig> Function BeginGetEvent() As Integer
        <PreserveSig> Function EndGetEvent() As Integer
        <PreserveSig> Function QueueEvent() As Integer
    End Interface
#End Region

#Region "Interface IMFPresentationDescriptor"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IMFPresentationDescriptor)>
    Private Interface IMFPresentationDescriptor
        Inherits IMFAttributes

        <PreserveSig> Shadows Function GetItem(<[In]> ByRef guidKey As Guid,
                                       <Out> ByRef pValue As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function GetItemType(<[In]> ByRef guidKey As Guid,
                                           <Out> ByRef pType As MF_ATTRIBUTE_TYPE) As Integer

        <PreserveSig> Shadows Function CompareItem(<[In]> ByRef guidKey As Guid,
                                           <[In]> ByRef pValue As PROPVARIANT,
                                           <Out, MarshalAs(UnmanagedType.Bool)> ByRef pbResult As Boolean) As Integer

        <PreserveSig> Shadows Function Compare(<[In], MarshalAs(UnmanagedType.Interface)> ByVal pTheirs As IMFAttributes,
                                       <[In]> ByVal MatchType As MF_ATTRIBUTES_MATCH_TYPE,
                                       <Out, MarshalAs(UnmanagedType.Bool)> ByRef pbResult As Boolean) As Integer

        <PreserveSig> Shadows Function GetUInt32(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As UInteger) As Integer

        <PreserveSig> Shadows Function GetUINT64(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As ULong) As Integer

        <PreserveSig> Shadows Function GetDouble(<[In]> ByRef guidKey As Guid,
                                         <Out> ByRef punValue As Double) As Integer

        <PreserveSig> Shadows Function GetGUID(<[In]> ByRef guidKey As Guid,
                                       <Out> ByRef pguidValue As Guid) As Integer

        <PreserveSig> Shadows Function GetStringLength(<[In]> ByRef guidKey As Guid,
                                               <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetString(<[In]> ByRef guidKey As Guid,
                                         <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef pwszValue As String,
                                         <[In]> ByVal cchBufSize As UInteger,
                                         <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetAllocatedString(<[In]> ByRef guidKey As Guid,
                                                  <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef ppwszValue As String,
                                                  <Out> ByRef pcchLength As UInteger) As Integer

        <PreserveSig> Shadows Function GetBlobSize(<[In]> ByRef guidKey As Guid,
                                           <Out> ByRef pcbBlobSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetBlob(<[In]> ByRef guidKey As Guid,
                                       <Out, MarshalAs(UnmanagedType.LPArray)> ByRef pBuf As Byte(),
                                       <[In]> ByVal cbBufSize As UInteger,
                                       <Out> ByRef pcbBlobSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetAllocatedBlob(<[In]> ByRef guidKey As Guid,
                                                <Out, MarshalAs(UnmanagedType.LPArray)> ByRef pBuf As Byte(),
                                                <Out> ByRef pcbSize As UInteger) As Integer

        <PreserveSig> Shadows Function GetUnknown(<[In]> ByRef guidKey As Guid,
                                          <[In]> ByRef riid As Guid,
                                          <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppv As Object) As Integer

        <PreserveSig> Shadows Function SetItem(<[In]> ByRef guidKey As Guid,
                                       <[In]> ByRef Value As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function DeleteItem(<[In]> ByRef guidKey As Guid) As Integer

        <PreserveSig> Shadows Function DeleteAllItems() As Integer

        <PreserveSig> Shadows Function SetUInt32(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal unValue As UInteger) As Integer

        <PreserveSig> Shadows Function SetUINT64(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal unValue As ULong) As Integer

        <PreserveSig> Shadows Function SetDouble(<[In]> ByRef guidKey As Guid,
                                         <[In]> ByVal fValue As Double) As Integer

        <PreserveSig> Shadows Function SetGUID(<[In]> ByRef guidKey As Guid,
                                       <[In]> ByRef guidValue As Guid) As Integer

        <PreserveSig> Shadows Function SetString(<[In]> ByRef guidKey As Guid,
                                         <[In], MarshalAs(UnmanagedType.LPWStr)> ByVal wszValue As String) As Integer

        <PreserveSig> Shadows Function SetBlob(<[In]> ByRef guidKey As Guid,
                                       <[In], MarshalAs(UnmanagedType.LPArray)> ByVal pBuf As Byte(),
                                       <[In]> ByVal cbBufSize As UInteger) As Integer

        <PreserveSig> Shadows Function SetUnknown(<[In]> ByRef guidKey As Guid,
                                          <[In], MarshalAs(UnmanagedType.Interface)> ByVal pUnknown As Object) As Integer

        <PreserveSig> Shadows Function LockStore() As Integer

        <PreserveSig> Shadows Function UnlockStore() As Integer

        <PreserveSig> Shadows Function GetCount(<Out> ByRef pcItems As UInteger) As Integer

        <PreserveSig> Shadows Function GetItemByIndex(<[In]> ByVal unIndex As UInteger,
                                              <[In]> ByRef pguidKey As Guid,
                                              <Out> ByRef pValue As PROPVARIANT) As Integer

        <PreserveSig> Shadows Function CopyAllItems(<Out, MarshalAs(UnmanagedType.Interface)> ByRef pDest As IMFAttributes) As Integer


        <PreserveSig> Function GetStreamDescriptorCount(<Out> ByRef pdwDescriptorCount As Integer) As Integer

        <PreserveSig> Function GetStreamDescriptorByIndex(<[In]> ByVal dwIndex As Integer,
                                                          <Out, MarshalAs(UnmanagedType.Bool)> ByRef pfSelected As Boolean,
                                                          <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppDescriptor As IMFStreamDescriptor) As Integer

        <PreserveSig> Function SelectStream() As Integer ' nur Dummyfunktion

        <PreserveSig> Function DeselectStream() As Integer ' nur Dummyfunktion

        <PreserveSig> Function Clone() As Integer ' nur Dummyfunktion

    End Interface
#End Region

End Class
